#version 330

const int MAX_BONES = 100;

layout(location = 0) in vec4 position;
layout(location = 1) in vec4 color;
layout(location = 2) in vec4 normal;
layout(location = 3) in vec2 texCoords;
layout(location = 4) in ivec4 bones;
layout(location = 5) in vec4 weights;

uniform mat4 MVP;
uniform mat4 boneTransformations[MAX_BONES];

// new
uniform mat4 modelMatrix;
uniform mat4 normalMatrix;
uniform vec4 cameraLocation;
uniform mat4 shadowTransformation;

smooth out vec4 theColor;
out vec3 vertexWorldSpace;
out vec3 vertexNormal;
out vec3 vertexToCam;
out vec4 SM_tex_coord;
// new

out vec2 textureCoords;

void main(){
	mat4 boneTransform  = boneTransformations[bones[0]] * weights[0];
	     boneTransform += boneTransformations[bones[1]] * weights[1];
		 boneTransform += boneTransformations[bones[2]] * weights[2];
		 boneTransform += boneTransformations[bones[3]] * weights[3];

	gl_Position = MVP * boneTransform * position;
	textureCoords = texCoords;

	theColor = color;
	vertexWorldSpace = (modelMatrix * boneTransform * position).xyz;
	vertexNormal = (normalMatrix * boneTransform * normal).xyz;
	vertexToCam = cameraLocation.xyz - vertexWorldSpace;

	// shadow texture coords in projected light space
	SM_tex_coord = shadowTransformation * modelMatrix * boneTransform * position;
}
